@@ -91,6 +91,7 @@ class cDocumentFolder:
 			return None
 		prescription = cDocument(aPK_obj = rows[0][0])
 		return prescription
+
 	#--------------------------------------------------------
 	def get_latest_mugshot(self):
 		cmd = u"SELECT pk_obj FROM blobs.v_latest_mugshot WHERE pk_patient = %s"
@@ -101,6 +102,7 @@ class cDocumentFolder:
 		return cDocumentPart(aPK_obj = rows[0][0])
 
 	latest_mugshot = property(get_latest_mugshot, lambda x:x)
+
 	#--------------------------------------------------------
 	def get_mugshot_list(self, latest_only=True):
 		if latest_only:
@@ -120,6 +122,7 @@ class cDocumentFolder:
 			"""
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': [self.pk_patient]}])
 		return rows
+
 	#--------------------------------------------------------
 	def get_doc_list(self, doc_type=None):
 		"""return flat list of document IDs"""
@@ -151,13 +154,15 @@ class cDocumentFolder:
 		for row in rows:
 			doc_ids.append(row[0])
 		return doc_ids
+
 	#--------------------------------------------------------
 	def get_visual_progress_notes(self, episodes=None, encounter=None):
 		return self.get_documents (
 			doc_type = DOCUMENT_TYPE_VISUAL_PROGRESS_NOTE,
-			episodes = episodes,
+			pk_episodes = episodes,
 			encounter = encounter
 		)
+
 	#--------------------------------------------------------
 	def get_unsigned_documents(self):
 		args = {'pat': self.pk_patient}
@@ -173,8 +178,9 @@ class cDocumentFolder:
 			ORDER BY clin_when DESC"""
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 		return [ cDocument(row = {'pk_field': 'pk_doc', 'idx': idx, 'data': r}) for r in rows ]
+
 	#--------------------------------------------------------
-	def get_documents(self, doc_type=None, episodes=None, encounter=None, order_by=None, exclude_unsigned=False):
+	def get_documents(self, doc_type=None, pk_episodes=None, encounter=None, order_by=None, exclude_unsigned=False, pk_types=None):
 		"""Return list of documents."""
 
 		args = {
@@ -191,9 +197,13 @@ class cDocumentFolder:
 			except (TypeError, ValueError):
 				where_parts.append(u'pk_type = (SELECT pk FROM blobs.doc_type WHERE name = %(type)s)')
 
-		if (episodes is not None) and (len(episodes) > 0):
-			where_parts.append(u'pk_episode IN %(epi)s')
-			args['epi'] = tuple(episodes)
+		if pk_types is not None:
+			where_parts.append(u'pk_type IN %(pk_types)s')
+			args['pk_types'] = tuple(pk_types)
+
+		if (pk_episodes is not None) and (len(pk_episodes) > 0):
+			where_parts.append(u'pk_episode IN %(epis)s')
+			args['epis'] = tuple(pk_episodes)
 
 		if encounter is not None:
 			where_parts.append(u'pk_encounter = %(enc)s')
@@ -266,7 +276,7 @@ class cDocumentPart(gmBusinessDBObject.cBusinessDBObject):
 	#--------------------------------------------------------
 	# retrieve data
 	#--------------------------------------------------------
-	def save_to_file(self, aChunkSize=0, filename=None, target_mime=None, target_extension=None, ignore_conversion_problems=False, directory=None):
+	def save_to_file(self, aChunkSize=0, filename=None, target_mime=None, target_extension=None, ignore_conversion_problems=False, directory=None, adjust_extension=False, conn=None):
 
 		if self._payload[self._idx['size']] == 0:
 			return None
@@ -281,13 +291,16 @@ class cDocumentPart(gmBusinessDBObject.cBusinessDBObject):
 			},
 			filename = filename,
 			chunk_size = aChunkSize,
-			data_size = self._payload[self._idx['size']]
+			data_size = self._payload[self._idx['size']],
+			conn = conn
 		)
-
 		if not success:
 			return None
 
 		if target_mime is None:
+			if filename.endswith(u'.dat'):
+				if adjust_extension:
+					return gmMimeLib.adjust_extension_by_mimetype(filename)
 			return filename
 
 		if target_extension is None:
@@ -296,7 +309,7 @@ class cDocumentPart(gmBusinessDBObject.cBusinessDBObject):
 		target_path, name = os.path.split(filename)
 		name, tmp = os.path.splitext(name)
 		target_fname = gmTools.get_unique_filename (
-			prefix = '%s-converted-' % name,
+			prefix = '%s-conv-' % name,
 			suffix = target_extension
 		)
 		_log.debug('attempting conversion: [%s] -> [<%s>:%s]', filename, target_mime, target_fname)
@@ -311,6 +324,9 @@ class cDocumentPart(gmBusinessDBObject.cBusinessDBObject):
 		if not ignore_conversion_problems:
 			return None
 
+		if filename.endswith(u'.dat'):
+			if adjust_extension:
+				filename = gmMimeLib.adjust_extension_by_mimetype(filename)
 		_log.warning('programmed to ignore conversion problems, hoping receiver can handle [%s]', filename)
 		return filename
 
@@ -416,6 +432,7 @@ insert into blobs.reviewed_doc_objs (
 		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
 
 		return True
+
 	#--------------------------------------------------------
 	def set_as_active_photograph(self):
 		if self._payload[self._idx['type']] != u'patient photograph':
@@ -431,6 +448,7 @@ insert into blobs.reviewed_doc_objs (
 		self._payload[self._idx['seq_idx']] = rows[0][0]
 		self._is_modified = True
 		self.save_payload()
+
 	#--------------------------------------------------------
 	def reattach(self, pk_doc=None):
 		if pk_doc == self._payload[self._idx['pk_doc']]:
@@ -480,6 +498,7 @@ insert into blobs.reviewed_doc_objs (
 			return False, msg
 
 		return True, ''
+
 	#--------------------------------------------------------
 	def format_single_line(self):
 		f_ext = u''
@@ -550,8 +569,9 @@ insert into blobs.reviewed_doc_objs (
 		# preserve original filename extension if available
 		suffix = '.dat'
 		if self._payload[self._idx['filename']] is not None:
-			tmp, suffix = os.path.splitext(self._payload[self._idx['filename']])
-			suffix = suffix.strip().replace(' ', '-').lower()
+			tmp, suffix = os.path.splitext (
+				gmTools.fname_sanitize(self._payload[self._idx['filename']]).lower()
+			)
 			if suffix == u'':
 				suffix = '.dat'
 
@@ -580,12 +600,12 @@ insert into blobs.reviewed_doc_objs (
 
 		if make_unique:
 			fname = gmTools.get_unique_filename (
-				prefix = '%s-' % fname,
+				prefix = '%s-' % gmTools.fname_sanitize(fname),
 				suffix = suffix,
 				tmp_dir = directory
 			)
 		else:
-			fname = os.path.join(gmTools.coalesce(directory, u''), fname + suffix)
+			fname = gmTools.fname_sanitize(os.path.join(gmTools.coalesce(directory, u''), fname + suffix))
 
 		return fname
 
@@ -601,38 +621,29 @@ def delete_document_part(part_pk=None, encounter_pk=None):
 	return
 
 #============================================================
-_sql_fetch_document_fields = u"""
-		SELECT
-			*,
-		COALESCE (
-			(SELECT array_agg(seq_idx) FROM blobs.doc_obj b_do WHERE b_do.fk_doc = b_vdm.pk_doc),
-			ARRAY[]::integer[]
-		)
-			AS seq_idx_list
-		FROM
-			blobs.v_doc_med b_vdm
-		WHERE
-			%s"""
+_sql_fetch_document_fields = u"SELECT * FROM blobs.v_doc_med b_vdm WHERE %s"
 
 class cDocument(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents one medical document."""
 
 	_cmd_fetch_payload = _sql_fetch_document_fields % u"pk_doc = %s"
 	_cmds_store_payload = [
-		u"""update blobs.doc_med set
+		u"""UPDATE blobs.doc_med SET
 				fk_type = %(pk_type)s,
 				fk_episode = %(pk_episode)s,
 				fk_encounter = %(pk_encounter)s,
 				fk_org_unit = %(pk_org_unit)s,
+				unit_is_receiver = %(unit_is_receiver)s,
 				clin_when = %(clin_when)s,
 				comment = gm.nullify_empty_string(%(comment)s),
-				ext_ref = gm.nullify_empty_string(%(ext_ref)s)
-			where
+				ext_ref = gm.nullify_empty_string(%(ext_ref)s),
+				fk_hospital_stay = %(pk_hospital_stay)s
+			WHERE
 				pk = %(pk_doc)s and
-				xmin = %(xmin_doc_med)s""",
-		u"""select xmin_doc_med from blobs.v_doc_med where pk_doc = %(pk_doc)s"""
-		]
-
+				xmin = %(xmin_doc_med)s
+			RETURNING
+				xmin AS xmin_doc_med"""
+	]
 	_updatable_fields = [
 		'pk_type',
 		'comment',
@@ -640,14 +651,18 @@ class cDocument(gmBusinessDBObject.cBusinessDBObject):
 		'ext_ref',
 		'pk_episode',
 		'pk_encounter',			# mainly useful when moving visual progress notes to their respective encounters
-		'pk_org_unit'
+		'pk_org_unit',
+		'unit_is_receiver',
+		'pk_hospital_stay'
 	]
+
 	#--------------------------------------------------------
 	def refetch_payload(self, ignore_changes=False, link_obj=None):
 		try: del self.__has_unreviewed_parts
 		except AttributeError: pass
 
 		return super(cDocument, self).refetch_payload(ignore_changes = ignore_changes, link_obj = link_obj)
+
 	#--------------------------------------------------------
 	def get_descriptions(self, max_lng=250):
 		"""Get document descriptions.
@@ -660,11 +675,13 @@ class cDocument(gmBusinessDBObject.cBusinessDBObject):
 			cmd = u"SELECT pk, substring(text from 1 for %s) FROM blobs.doc_desc WHERE fk_doc=%%s" % max_lng
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': [self.pk_obj]}])
 		return rows
+
 	#--------------------------------------------------------
 	def add_description(self, description=None):
 		cmd = u"insert into blobs.doc_desc (fk_doc, text) values (%s, %s)"
 		gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': [self.pk_obj, description]}])
 		return True
+
 	#--------------------------------------------------------
 	def update_description(self, pk=None, description=None):
 		cmd = u"update blobs.doc_desc set text = %(desc)s where fk_doc = %(doc)s and pk = %(pk_desc)s"
@@ -672,11 +689,13 @@ class cDocument(gmBusinessDBObject.cBusinessDBObject):
 			{'cmd': cmd, 'args': {'doc': self.pk_obj, 'pk_desc': pk, 'desc': description}}
 		])
 		return True
+
 	#--------------------------------------------------------
 	def delete_description(self, pk=None):
 		cmd = u"delete from blobs.doc_desc where fk_doc = %(doc)s and pk = %(desc)s"
 		gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': {'doc': self.pk_obj, 'desc': pk}}])
 		return True
+
 	#--------------------------------------------------------
 	def _get_parts(self):
 		cmd = _sql_fetch_document_part_fields % u"pk_doc = %s ORDER BY seq_idx"
@@ -684,6 +703,7 @@ class cDocument(gmBusinessDBObject.cBusinessDBObject):
 		return [ cDocumentPart(row = {'pk_field': 'pk_obj', 'idx': idx, 'data': r}) for r in rows ]
 
 	parts = property(_get_parts, lambda x:x)
+
 	#--------------------------------------------------------
 	def add_part(self, file=None, link_obj=None):
 		"""Add a part to the document."""
@@ -715,6 +735,7 @@ class cDocument(gmBusinessDBObject.cBusinessDBObject):
 		new_part.save_payload(conn = link_obj)
 
 		return new_part
+
 	#--------------------------------------------------------
 	def add_parts_from_files(self, files=None, reviewer=None):
 
@@ -738,16 +759,21 @@ class cDocument(gmBusinessDBObject.cBusinessDBObject):
 					return (False, msg, filename)
 
 		return (True, '', new_parts)
+
 	#--------------------------------------------------------
-	def save_parts_to_files(self, export_dir=None, chunksize=0):
+	def save_parts_to_files(self, export_dir=None, chunksize=0, conn=None):
 		fnames = []
 		for part in self.parts:
-			fname = part.save_to_file(aChunkSize = chunksize)
-			if export_dir is not None:
-				shutil.move(fname, export_dir)
-				fname = os.path.join(export_dir, os.path.split(fname)[1])
+			fname = part.save_to_file(aChunkSize = chunksize, directory = export_dir, conn = conn)
+#			if export_dir is not None:
+#				shutil.move(fname, export_dir)
+#				fname = os.path.join(export_dir, os.path.split(fname)[1])
+			if fname is None:
+				_log.error(u'cannot export document part [%s]', part)
+				continue
 			fnames.append(fname)
 		return fnames
+
 	#--------------------------------------------------------
 	def _get_has_unreviewed_parts(self):
 		try:
@@ -832,6 +858,27 @@ class cDocument(gmBusinessDBObject.cBusinessDBObject):
 			parts = _('1 part')
 		else:
 			parts = _('%s parts') % part_count
+		org = u''
+		if self._payload[self._idx['unit']] is not None:
+			if self._payload[self._idx['unit_is_receiver']]:
+				org = _(' Receiver: %s @ %s\n') % (
+					self._payload[self._idx['unit']],
+					self._payload[self._idx['organization']]
+				)
+			else:
+				org = _(' Sender: %s @ %s\n') % (
+					self._payload[self._idx['unit']],
+					self._payload[self._idx['organization']]
+				)
+		stay = u''
+		if self._payload[self._idx['pk_hospital_stay']] is not None:
+			stay = _(u'Hospital stay') + u': %s\n' % self.hospital_stay.format (
+				left_margin = 0,
+				include_procedures = False,
+				include_docs = False,
+				include_episode = False
+			)
+
 		txt = _(
 			'%s (%s)   #%s\n'
 			' Created: %s\n'
@@ -840,6 +887,7 @@ class cDocument(gmBusinessDBObject.cBusinessDBObject):
 			'%s'
 			'%s'
 			'%s'
+			'%s'
 		) % (
 			self._payload[self._idx['l10n_type']],
 			parts,
@@ -848,17 +896,44 @@ class cDocument(gmBusinessDBObject.cBusinessDBObject):
 			self._payload[self._idx['episode']],
 			gmTools.coalesce(self._payload[self._idx['health_issue']], u'', _(' Health issue: %s\n')),
 			gmTools.coalesce(self._payload[self._idx['ext_ref']], u'', _(' External reference: %s\n')),
-			gmTools.coalesce(self._payload[self._idx['unit']], u'', _(' Organization: %%s @ %s\n') % self._payload[self._idx['organization']]),
+			org,
+			stay,
 			gmTools.coalesce(self._payload[self._idx['comment']], u'', u' %s')
 		)
+
 		return txt
 
+	#--------------------------------------------------------
+	def _get_hospital_stay(self):
+		if self._payload[self._idx['pk_hospital_stay']] is None:
+			return None
+		from Gnumed.business import gmEMRStructItems
+		return gmEMRStructItems.cHospitalStay(self._payload[self._idx['pk_hospital_stay']])
+
+	hospital_stay = property(_get_hospital_stay, lambda x:x)
+
 	#--------------------------------------------------------
 	def _get_org_unit(self):
+		if self._payload[self._idx['pk_org_unit']] is None:
+			return None
 		return gmOrganization.cOrgUnit(self._payload[self._idx['pk_org_unit']])
 
 	org_unit = property(_get_org_unit, lambda x:x)
 
+	#--------------------------------------------------------
+	def _get_procedures(self):
+		from gmEMRStructItems import get_procedures4document
+		return get_procedures4document(pk_document = self.pk_obj)
+
+	procedures = property(_get_procedures, lambda x:x)
+
+	#--------------------------------------------------------
+	def _get_bills(self):
+		from gmBilling import get_bills4document
+		return get_bills4document(pk_document = self.pk_obj)
+
+	bills = property(_get_bills, lambda x:x)
+
 #------------------------------------------------------------
 def create_document(document_type=None, encounter=None, episode=None, link_obj=None):
 	"""Returns new document instance or raises an exception."""
@@ -886,13 +961,22 @@ def create_document(document_type=None, encounter=None, episode=None, link_obj=N
 	return doc
 
 #------------------------------------------------------------
-def search_for_documents(patient_id=None, type_id=None, external_reference=None):
+def search_for_documents(patient_id=None, type_id=None, external_reference=None, pk_episode=None, pk_types=None):
 	"""Searches for documents with the given patient and type ID."""
-	if patient_id is None:
-		raise ValueError('need patient id to search for document')
 
-	args = {'pat_id': patient_id, 'type_id': type_id, 'ref': external_reference}
-	where_parts = [u'pk_patient = %(pat_id)s']
+	if (patient_id is None) and (pk_episode is None):
+		raise ValueError('need patient_id or pk_episode to search for document')
+
+	where_parts = []
+	args = {
+		'pat_id': patient_id,
+		'type_id': type_id,
+		'ref': external_reference,
+		'pk_epi': pk_episode
+	}
+
+	if patient_id is not None:
+		where_parts.append(u'pk_patient = %(pat_id)s')
 
 	if type_id is not None:
 		where_parts.append(u'pk_type = %(type_id)s')
@@ -900,6 +984,13 @@ def search_for_documents(patient_id=None, type_id=None, external_reference=None)
 	if external_reference is not None:
 		where_parts.append(u'ext_ref = %(ref)s')
 
+	if pk_episode is not None:
+		where_parts.append(u'pk_episode = %(pk_epi)s')
+
+	if pk_types is not None:
+		where_parts.append(u'pk_type IN %(pk_types)s')
+		args['pk_types'] = tuple(pk_types)
+
 	cmd = _sql_fetch_document_fields % u' AND '.join(where_parts)
 	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 	return [ cDocument(row = {'data': r, 'idx': idx, 'pk_field': 'pk_doc'}) for r in rows ]
@@ -976,6 +1067,7 @@ class cDocumentType(gmBusinessDBObject.cBusinessDBObject):
 			return False
 
 		return self.refetch_payload()
+
 #------------------------------------------------------------
 def get_document_types():
 	rows, idx = gmPG2.run_ro_queries (
@@ -987,6 +1079,7 @@ def get_document_types():
 		row_def = {'pk_field': 'pk_doc_type', 'idx': idx, 'data': row}
 		doc_types.append(cDocumentType(row = row_def))
 	return doc_types
+
 #------------------------------------------------------------
 def get_document_type_pk(document_type=None):
 	args = {'typ': document_type.strip()}
@@ -1001,6 +1094,14 @@ def get_document_type_pk(document_type=None):
 		return None
 
 	return rows[0]['pk']
+
+#------------------------------------------------------------
+def map_types2pk(document_types=None):
+	args = {'types': tuple(document_types)}
+	cmd = u'SELECT pk_doc_type, coalesce(l10n_type, type) as desc FROM blobs.v_doc_type WHERE l10n_type IN %(types)s OR type IN %(types)s'
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+	return rows
+
 #------------------------------------------------------------
 def create_document_type(document_type=None):
 	# check for potential dupes:
@@ -1016,6 +1117,7 @@ def create_document_type(document_type=None):
 			return_data = True
 		)
 	return cDocumentType(aPK_obj = rows[0][0])
+
 #------------------------------------------------------------
 def delete_document_type(document_type=None):
 	if document_type['is_in_use']:
@@ -1027,6 +1129,7 @@ def delete_document_type(document_type=None):
 		}]
 	)
 	return True
+
 #------------------------------------------------------------
 def get_ext_ref():
 	"""This needs *considerably* more smarts."""
@@ -1095,6 +1198,7 @@ if __name__ == '__main__':
 		return
 	#--------------------------------------------------------
 	def test_get_documents():
+
 		doc_folder = cDocumentFolder(aPKey=12)
 
 		#photo = doc_folder.get_latest_mugshot()
@@ -1133,7 +1237,7 @@ if __name__ == '__main__':
 
 	#test_doc_types()
 	#test_adding_doc_part()
-	#test_get_documents()
-	test_get_useful_filename()
+	test_get_documents()
+	#test_get_useful_filename()
 
 #	print get_ext_ref()
