@@ -30,6 +30,7 @@ from Gnumed.pycommon import gmDateTime
 from Gnumed.pycommon import gmCfg2
 
 from Gnumed.business import gmDocuments
+from Gnumed.business import gmKeywordExpansion
 
 
 _log = logging.getLogger('gm.exp_area')
@@ -47,10 +48,7 @@ class cExportItem(gmBusinessDBObject.cBusinessDBObject):
 	_cmd_fetch_payload = _SQL_get_export_items % u"pk_export_item = %s"
 	_cmds_store_payload = [
 		u"""UPDATE clin.export_item SET
-				fk_identity = CASE
-					WHEN %(pk_doc_obj)s IS NULL THEN %(pk_identity)s
-					ELSE NULL
-				END,
+				fk_identity = %(pk_identity)s,
 				created_by = gm.nullify_empty_string(%(created_by)s),
 				created_when = %(created_when)s,
 				designation = gm.nullify_empty_string(%(designation)s),
@@ -80,6 +78,27 @@ class cExportItem(gmBusinessDBObject.cBusinessDBObject):
 		u'filename'
 	]
 	#--------------------------------------------------------
+	def __init__(self, aPK_obj=None, row=None, link_obj=None):
+		super(cExportItem, self).__init__(aPK_obj = aPK_obj, row = row, link_obj = link_obj)
+		# force auto-healing if need be
+		if self._payload[self._idx['pk_identity_raw_needs_update']]:
+			_log.warning (
+				u'auto-healing export item [%s] from identity [%s] to [%s] because of document part [%s] seems necessary',
+				self._payload[self._idx['pk_export_item']],
+				self._payload[self._idx['pk_identity_raw']],
+				self._payload[self._idx['pk_identity']],
+				self._payload[self._idx['pk_doc_obj']]
+			)
+			if self._payload[self._idx['pk_doc_obj']] is None:
+				_log.error(u'however, .fk_doc_obj is NULL, which should not happen, leaving things alone for manual inspection')
+				return
+			# only flag ourselves as modified, do not actually
+			# modify any values, better safe than sorry
+			self._is_modified = True
+			self.save()
+			self.refetch_payload(ignore_changes = False, link_obj = link_obj)
+
+	#--------------------------------------------------------
 #	def format(self):
 #		return u'%s' % self
 	#--------------------------------------------------------
@@ -120,8 +139,8 @@ class cExportItem(gmBusinessDBObject.cBusinessDBObject):
 			return part.save_to_file (
 				aChunkSize = aChunkSize,
 				filename = filename,
-				ignore_conversion_problems = True
-				#, directory = directory
+				ignore_conversion_problems = True,
+				adjust_extension = True
 			)
 
 		# data in export area table
@@ -137,10 +156,12 @@ class cExportItem(gmBusinessDBObject.cBusinessDBObject):
 			chunk_size = aChunkSize,
 			data_size = self._payload[self._idx['size']]
 		)
-
 		if not success:
 			return None
 
+		if filename.endswith(u'.dat'):
+			return gmMimeLib.adjust_extension_by_mimetype(filename)
+
 		return filename
 
 	#--------------------------------------------------------
@@ -168,8 +189,9 @@ class cExportItem(gmBusinessDBObject.cBusinessDBObject):
 		# preserve original filename extension if available
 		suffix = '.dat'
 		if self._payload[self._idx['filename']] is not None:
-			tmp, suffix = os.path.splitext(self._payload[self._idx['filename']])
-			suffix = suffix.strip().replace(' ', '-').lower()
+			tmp, suffix = os.path.splitext (
+				gmTools.fname_sanitize(self._payload[self._idx['filename']]).lower()
+			)
 			if suffix == u'':
 				suffix = '.dat'
 
@@ -190,6 +212,7 @@ class cExportItem(gmBusinessDBObject.cBusinessDBObject):
 		return gmDocuments.cDocumentPart(aPK_obj = self._payload[self._idx['pk_doc_obj']])
 
 	document_part = property(_get_doc_part, lambda x:x)
+
 	#--------------------------------------------------------
 	def _get_is_print_job(self):
 		return self._payload[self._idx['designation']] == PRINT_JOB_DESIGNATION
@@ -202,6 +225,7 @@ class cExportItem(gmBusinessDBObject.cBusinessDBObject):
 		self.save()
 
 	is_print_job = property(_get_is_print_job, _set_is_print_job)
+
 #------------------------------------------------------------
 def get_export_items(order_by=None, pk_identity=None, designation=None):
 
@@ -212,6 +236,8 @@ def get_export_items(order_by=None, pk_identity=None, designation=None):
 	where_parts = []
 	if pk_identity is not None:
 		where_parts.append(u'pk_identity = %(pat)s')
+		# note that invalidly linked items will be
+		# auto-healed when instantiated
 	if designation is None:
 		where_parts.append(u"designation IS DISTINCT FROM %(desig)s")
 	else:
@@ -250,10 +276,7 @@ def create_export_item(description=None, pk_identity=None, pk_doc_obj=None, file
 		) VALUES (
 			gm.nullify_empty_string(%(desc)s),
 			%(pk_obj)s,
-			(CASE
-				WHEN %(pk_obj)s IS NULL THEN %(pk_pat)s
-				ELSE NULL::integer
-			END),
+			%(pk_pat)s,
 			(CASE
 				WHEN %(pk_obj)s IS NULL THEN %(fname)s::bytea
 				ELSE NULL::bytea
@@ -282,6 +305,7 @@ _html_start = u"""<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
 <html>
 <head>
 <meta http-equiv="content-type" content="text/html; charset=UTF-8">
+<link rel="icon" type="image/x-icon" href="gnumed.ico">
 <title>%(html_title_header)s %(html_title_patient)s</title>
 </head>
 <body>
@@ -301,6 +325,7 @@ _html_start = u"""<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
 	<li><a href="./">%(browse_root)s</a></li>
 	<li><a href="documents/">%(browse_docs)s</a></li>
 	%(browse_dicomdir)s
+	%(run_dicom_viewer)s
 </ul>
 
 <ul>
@@ -327,11 +352,20 @@ _html_end = u"""
 """
 
 
-_autorun_inf = (
-	u'[AutoRun]\r\n'						# needs \r\n for Windows
-	u'label=%s\r\n'							# patient name/DOB
+_autorun_inf = (							# needs \r\n for Windows
+	u'[AutoRun.Amd64]\r\n'					# 64 bit
+	u'label=%(label)s\r\n'					# patient name/DOB
 	u'shellexecute=index.html\r\n'
-	u'action=%s\r\n'						# % _('Browse patient data')
+	u'action=%(action)s\r\n'				# % _('Browse patient data')
+	u'%(icon)s\r\n'							# "icon=gnumed.ico" or ""
+	u'UseAutoPlay=1\r\n'
+	u'\r\n'
+	u'[AutoRun]\r\n'						# 32 bit
+	u'label=%(label)s\r\n'					# patient name/DOB
+	u'shellexecute=index.html\r\n'
+	u'action=%(action)s\r\n'				# % _('Browse patient data')
+	u'%(icon)s\r\n'							# "icon=gnumed.ico" or ""
+	u'UseAutoPlay=1\r\n'
 	u'\r\n'
 	u'[Content]\r\n'
 	u'PictureFiles=yes\r\n'
@@ -343,7 +377,6 @@ _autorun_inf = (
 	u'\r\n'
 	u'[unused]\r\n'
 	u'open=requires explicit executable\r\n'
-	u'icon=use standard icon for storage unit\r\n'
 )
 
 
@@ -356,6 +389,8 @@ u'CreationDate=%s\r\n'
 u'PID=%s\r\n'
 u'EMR=GNUmed\r\n'
 u'Version=%s\r\n'
+u'#StudyDate=\r\n'
+u'#VNRInfo=<body part>\r\n'
 u'\r\n'
 u'# name format: lastnames, firstnames\r\n'
 u'# date format: YYYY-MM-DD (ISO 8601)\r\n'
@@ -502,7 +537,26 @@ class cExportArea(object):
 		return cExportItem(row = {'data': r, 'idx': idx, 'pk_field': 'pk_export_item'})
 
 	#--------------------------------------------------------
-	def export(self, base_dir=None, items=None, with_metadata=True, expand_compressed=False):
+	def dump_items_to_disk(self, base_dir=None, items=None):
+		if items is None:
+			items = self.items
+
+		if len(items) == 0:
+			return None
+
+		if base_dir is None:
+			from Gnumed.business.gmPerson import cPatient
+			pat = cPatient(aPK_obj = self.__pk_identity)
+			base_dir = gmTools.mk_sandbox_dir(prefix = u'exp-%s-' % pat.dirname)
+		_log.debug('dumping export items to: %s', base_dir)
+
+		gmTools.mkdir(base_dir)
+		for item in items:
+			item.save_to_file(directory = base_dir)
+		return base_dir
+
+	#--------------------------------------------------------
+	def export(self, base_dir=None, items=None, expand_compressed=False):
 
 		if items is None:
 			items = self.items
@@ -510,14 +564,20 @@ class cExportArea(object):
 		if len(items) == 0:
 			return None
 
+		media_base_dir = base_dir
+
 		from Gnumed.business.gmPerson import cPatient
 		pat = cPatient(aPK_obj = self.__pk_identity)
-		if base_dir is None:
-			base_dir = gmTools.mk_sandbox_dir(prefix = u'exp-%s-' % pat.dirname)
-		_log.debug('base dir: %s', base_dir)
+		if media_base_dir is None:
+			media_base_dir = gmTools.mk_sandbox_dir(prefix = u'exp-%s-' % pat.dirname)
+		_log.debug('patient media base dir: %s', media_base_dir)
 
-		doc_dir = os.path.join(base_dir, r'documents')
-		gmTools.mkdir(doc_dir)
+		doc_dir = os.path.join(media_base_dir, r'documents')
+		if os.path.isdir(doc_dir):
+			index_existing_docs = True
+		else:
+			index_existing_docs = False
+			gmTools.mkdir(doc_dir)
 
 		_html_start_data = {
 			u'html_title_header': _('Patient data for'),
@@ -531,24 +591,39 @@ class cExportArea(object):
 			u'docs_title': _(u'Documents'),
 			u'browse_root': _(u'browse storage medium'),
 			u'browse_docs': _(u'browse documents area'),
-			u'browse_dicomdir': u''
+			u'browse_dicomdir': u'',
+			u'run_dicom_viewer': u''
 		}
 
 		mugshot = pat.document_folder.latest_mugshot
 		if mugshot is not None:
-			_html_start_data['mugshot_url'] = mugshot.save_to_file(directory = doc_dir)
+			_html_start_data['mugshot_url'] = mugshot.save_to_file(directory = doc_dir, adjust_extension = True)
 			_html_start_data['mugshot_alt'] =_('patient photograph from %s') % gmDateTime.pydt_strftime(mugshot['date_generated'], '%B %Y')
 			_html_start_data['mugshot_title'] = gmDateTime.pydt_strftime(mugshot['date_generated'], '%B %Y')
 
+		if u'DICOMDIR' in os.listdir(media_base_dir):
+			_html_start_data[u'browse_dicomdir'] = u'<li><a href="./DICOMDIR">%s</a></li>' % _(u'show DICOMDIR file')
+			# copy DWV into target dir
+			dwv_src_dir = os.path.join(gmTools.gmPaths().local_base_dir, u'dwv4export')
+			if not os.path.isdir(dwv_src_dir):
+				dwv_src_dir = os.path.join(gmTools.gmPaths().system_app_data_dir, u'dwv4export')
+			if os.path.isdir(dwv_src_dir):
+				dwv_target_dir = os.path.join(media_base_dir, u'dwv')
+				gmTools.rmdir(dwv_target_dir)
+				try:
+					shutil.copytree(dwv_src_dir, dwv_target_dir)
+					_html_start_data[u'run_dicom_viewer'] = u'<li><a href="./dwv/viewers/mobile-local/index.html">%s</a></li>' % _(u'run Radiology Images (DICOM) Viewer')
+				except shutil.Error, OSError:
+					_log.exception('cannot include DWV, skipping')
+
 		# index.html
-		idx_fname = os.path.join(base_dir, u'index.html')
+		# - header
+		idx_fname = os.path.join(media_base_dir, u'index.html')
 		idx_file = io.open(idx_fname, mode = u'wt', encoding = u'utf8')
-		# header
-		existing_files = os.listdir(base_dir)
-		if u'DICOMDIR' in existing_files:
-			_html_start_data[u'browse_dicomdir'] = u'	<li><a href="./DICOMDIR">browse DICOMDIR</a></li>'
 		idx_file.write(_html_start % _html_start_data)
-		# middle (side effect ! -> exports items into files ...)
+		# - middle (side effect ! -> exports items into files ...)
+		existing_docs = os.listdir(doc_dir)		# get them now, or else we will include the to-be-exported items
+		# - export items
 		for item in items:
 			item_path = item.save_to_file(directory = doc_dir)
 			item_fname = os.path.split(item_path)[1]
@@ -556,7 +631,13 @@ class cExportArea(object):
 				item_fname,
 				gmTools.html_escape_string(item['description'])
 			))
-		# footer
+		# - preexisting documents
+		for doc_fname in existing_docs:
+			idx_file.write(_html_list_item % (
+				doc_fname,
+				gmTools.html_escape_string(_(u'other: %s') % doc_fname)
+			))
+		# - footer
 		_cfg = gmCfg2.gmCfgData()
 		from Gnumed.business.gmPraxis import gmCurrentPraxisBranch
 		prax = gmCurrentPraxisBranch()
@@ -586,29 +667,44 @@ class cExportArea(object):
 		idx_file.close()
 
 		# start.html (just a copy of index.html, really ;-)
-		start_fname = os.path.join(base_dir, u'start.html')
+		start_fname = os.path.join(media_base_dir, u'start.html')
 		try:
 			shutil.copy2(idx_fname, start_fname)
 		except Exception:
 			_log.exception('cannot copy %s to %s', idx_fname, start_fname)
 
 		# autorun.inf
-		name = pat.active_name
-		last = name['lastnames'][:14]
-		first = name['firstnames'][:min(14, 18 - len(last))]
-		label = ((u'%s%s%s' % (
-			u'%s,%s' % (last, first),
-			gmTools.coalesce(pat['gender'], u'', u' (%s)'),
-			pat.get_formatted_dob(format = ' %Y%m%d', none_string = u'', honor_estimation = False)
-		)).strip())[:32]		# max 32 chars, supposedly ASCII, but LATIN1 works pretty well
-		action = _('Browse patient data')
-		autorun_fname = os.path.join(base_dir, u'autorun.inf')
-		autorun_file = io.open(autorun_fname, mode = u'wt', encoding = u'utf8')
-		autorun_file.write(_autorun_inf % (label, action))
+		autorun_dict = {}
+		autorun_dict['label'] = self._compute_autorun_inf_label(pat)
+		autorun_dict['action'] = _('Browse patient data')
+		autorun_dict['icon'] = u''
+		media_icon_kwd = u'$$gnumed_patient_media_export_icon'
+		media_icon_kwd_exp = gmKeywordExpansion.get_expansion (
+			keyword = media_icon_kwd,
+			textual_only = False,
+			binary_only = True
+		)
+		icon_tmp_file = media_icon_kwd_exp.save_to_file (
+			target_mime = u'image/x-icon',
+			target_extension = u'.ico',
+			ignore_conversion_problems = True
+		)
+		if icon_tmp_file is None:
+			_log.debug(u'cannot retrieve <%s>', media_icon_kwd)
+		else:
+			media_icon_fname = os.path.join(media_base_dir, u'gnumed.ico')
+			try:
+				shutil.move(icon_tmp_file, media_icon_fname)
+				autorun_dict['icon'] = u'icon=gnumed.ico'
+			except Exception:
+				_log.exception('cannot move %s to %s', icon_tmp_file, media_icon_fname)
+		autorun_fname = os.path.join(media_base_dir, u'autorun.inf')
+		autorun_file = io.open(autorun_fname, mode = 'wt', encoding = 'cp1252', errors = 'replace')
+		autorun_file.write(_autorun_inf % autorun_dict)
 		autorun_file.close()
 
 		# cd.inf
-		cd_inf_fname = os.path.join(base_dir, u'cd.inf')
+		cd_inf_fname = os.path.join(media_base_dir, u'cd.inf')
 		cd_inf_file = io.open(cd_inf_fname, mode = u'wt', encoding = u'utf8')
 		cd_inf_file.write(_cd_inf % (
 			pat['lastnames'],
@@ -623,7 +719,7 @@ class cExportArea(object):
 		cd_inf_file.close()
 
 		# README
-		readme_fname = os.path.join(base_dir, u'README')
+		readme_fname = os.path.join(media_base_dir, u'README')
 		readme_file = io.open(readme_fname, mode = u'wt', encoding = u'utf8')
 		readme_file.write(_README % (
 			pat.get_description_gender(with_nickname = False) + u', ' + _(u'born') + u' ' + pat.get_formatted_dob('%Y %B %d')
@@ -631,14 +727,44 @@ class cExportArea(object):
 		readme_file.close()
 
 		# patient demographics as GDT/XML/VCF
-		pat.export_as_gdt(filename = os.path.join(base_dir, u'patient.gdt'))
-		pat.export_as_xml_linuxmednews(filename = os.path.join(base_dir, u'patient.xml'))
-		pat.export_as_vcard(filename = os.path.join(base_dir, u'patient.vcf'))
+		pat.export_as_gdt(filename = os.path.join(media_base_dir, u'patient.gdt'))
+		pat.export_as_xml_linuxmednews(filename = os.path.join(media_base_dir, u'patient.xml'))
+		pat.export_as_vcard(filename = os.path.join(media_base_dir, u'patient.vcf'))
 
 		# praxis VCF
-		shutil.move(prax.vcf, os.path.join(base_dir, u'praxis.vcf'))
+		shutil.move(prax.vcf, os.path.join(media_base_dir, u'praxis.vcf'))
 
-		return base_dir
+		return media_base_dir
+
+	#--------------------------------------------------------
+	def _compute_autorun_inf_label(self, patient):
+		LABEL_MAX_LEN = 32
+		dob = patient.get_formatted_dob(format = ' %Y%m%d', none_string = u'', honor_estimation = False)
+		if dob == u'':
+			gender_template = u' (%s)'
+		else:
+			gender_template = u' %s'
+		gender = gmTools.coalesce(patient['gender'], u'', gender_template)
+		name_max_len = LABEL_MAX_LEN - len(gender) - len(dob)			# they already include appropriate padding
+		name = patient.active_name
+		last = name['lastnames'].strip()
+		first = name['firstnames'].strip()
+		len_last = len(last)
+		len_first = len(first)
+		while (len_last + len_first + 1) > name_max_len:
+			if len_first > 6:
+				len_first -= 1
+				if first[len_first - 1] == u' ':
+					len_first -= 1
+				continue
+			len_last -= 1
+			if last[len_last - 1] == u' ':
+				len_last -= 1
+		last = last[:len_last].strip().upper()
+		first = first[:len_first].strip()
+		# max 32 chars, supposedly ASCII, but CP1252 likely works pretty well
+		label = ((u'%s %s%s%s' % (last, first, dob,	gender)).strip())[:32]
+		return label
 
 	#--------------------------------------------------------
 	# properties
@@ -681,6 +807,7 @@ if __name__ == '__main__':
 		item['pk_doc_obj'] = 1
 		item.save()
 		print item
+
 	#---------------------------------------
 	def test_export_area():
 		exp = cExportArea(12)
@@ -690,7 +817,55 @@ if __name__ == '__main__':
 		prax = gmPraxis.gmCurrentPraxisBranch(branch = gmPraxis.cPraxisBranch(1))
 		print prax
 		print prax.branch
-		print exp.export(with_metadata = True)
+		print exp.export()
+
+	#---------------------------------------
+	def test_label():
+
+		from Gnumed.business.gmPerson import cPatient
+		from Gnumed.business.gmPersonSearch import ask_for_patient
+
+		#while ask_for_patient() is not None:
+		pat_min = 1
+		pat_max = 100
+		try:
+			pat_min = int(sys.argv[2])
+			pat_max = int(sys.argv[3])
+		except:
+			pass
+		cPatient(aPK_obj = pat_min)
+		f = io.open('x-auto_inf_labels.txt', mode = 'w', encoding = 'utf8')
+		f.write(u'--------------------------------\n')
+		f.write(u'12345678901234567890123456789012\n')
+		f.write(u'--------------------------------\n')
+		for pat_id in range(pat_min, pat_max):
+			try:
+				exp_area = cExportArea(pat_id)
+				pat = cPatient(aPK_obj = pat_id)
+			except:
+				continue
+			f.write(exp_area._compute_autorun_inf_label(pat) + u'\n')
+		f.close()
+		return
+
 	#---------------------------------------
 	#test_export_items()
-	test_export_area()
+	#test_export_area()
+	test_label()
+
+	sys.exit(0)
+
+#============================================================
+# CDROM "run.bat":
+#
+#@echo off
+#
+#if defined ProgramFiles(x86) (
+#    ::64-bit
+#    start /B x64\mdicom.exe /scan .
+#) else (
+#    ::32-bit
+#    start /B win32\mdicom.exe /scan .
+#)
+#
+#--------------------------------------------------
