@@ -192,6 +192,7 @@ class cFormTemplate(gmBusinessDBObject.cBusinessDBObject):
 		return rows[0][0]
 
 	template_data = property(_get_template_data, lambda x:x)
+
 	#--------------------------------------------------------
 	def save_to_file(self, filename=None, chunksize=0):
 		"""Export form template from database into file."""
@@ -229,6 +230,7 @@ class cFormTemplate(gmBusinessDBObject.cBusinessDBObject):
 			return None
 
 		return filename
+
 	#--------------------------------------------------------
 	def update_template_from_file(self, filename=None):
 		gmPG2.file2bytea (
@@ -238,6 +240,7 @@ class cFormTemplate(gmBusinessDBObject.cBusinessDBObject):
 		)
 		# adjust for xmin change
 		self.refetch_payload()
+
 	#--------------------------------------------------------
 	def instantiate(self):
 		fname = self.save_to_file()
@@ -554,6 +557,7 @@ class gmOOoConnector(gmBorg.cBorg):
 		return self.__desktop
 
 	desktop = property(_get_desktop, lambda x:x)
+
 #------------------------------------------------------------
 class cOOoLetter(object):
 
@@ -718,7 +722,7 @@ class cFormEngine(object):
 #		# some forms may not have values ...
 #		if params is None:
 #			params = {}
-#		patient_clinical = self.patient.get_emr()
+#		patient_clinical = self.patient.emr
 #		encounter = patient_clinical.active_encounter['pk_encounter']
 #		# FIXME: get_active_episode is no more
 #		#episode = patient_clinical.get_active_episode()['pk_episode']
@@ -1169,6 +1173,8 @@ class cLaTeXForm(cFormEngine):
 		mimetypes = [
 			u'application/x-latex',
 			u'application/x-tex',
+			u'text/latex',
+			u'text/tex',
 			u'text/plain'
 		]
 
