@@ -58,6 +58,7 @@ u_currency_yen = u'\u00A5'					# Yen sign
 u_right_double_angle_quote = u'\u00AB'		# <<
 u_registered_trademark = u'\u00AE'
 u_plus_minus = u'\u00B1'
+u_superscript_one = u'\u00B9'				# ^1
 u_left_double_angle_quote = u'\u00BB'		# >>
 u_one_quarter = u'\u00BC'
 u_one_half = u'\u00BD'
@@ -67,6 +68,7 @@ u_greek_ALPHA = u'\u0391'
 u_greek_alpha = u'\u03b1'
 u_greek_OMEGA = u'\u03A9'
 u_greek_omega = u'\u03c9'
+u_dagger = u'\u2020'
 u_triangular_bullet = u'\u2023'					# triangular bullet  (>)
 u_ellipsis = u'\u2026'							# ...
 u_euro = u'\u20AC'								# EURO sign
@@ -77,19 +79,22 @@ u_up_arrow = u'\u2191'
 u_arrow2right = u'\u2192'						# -->
 u_down_arrow = u'\u2193'
 u_left_arrow_with_tail = u'\u21a2'				# <--<
+u_arrow2right_from_bar = u'\u21a6'				# |->
 u_arrow2right_until_vertical_bar = u'\u21e5'	# -->|
 u_sum = u'\u2211'								# sigma
 u_almost_equal_to = u'\u2248'					# approximately / nearly / roughly
 u_corresponds_to = u'\u2258'
 u_infinity = u'\u221E'
+u_arrow2right_until_vertical_bar2 = u'\u2b72'	# -->|
 u_diameter = u'\u2300'
 u_checkmark_crossed_out = u'\u237B'
 u_box_vert_left = u'\u23b8'
 u_box_vert_right = u'\u23b9'
 u_box_horiz_single = u'\u2500'				# -
 u_box_vert_light = u'\u2502'
+u_box_horiz_light_3dashes = u'\u2504'		# ...
 u_box_vert_light_4dashes = u'\u2506'
-u_box_horiz_4dashes = u'\u2508'
+u_box_horiz_4dashes = u'\u2508'				# ....
 u_box_T_right = u'\u251c'
 u_box_T_down = u'\u252c'
 u_box_T_up = u'\u2534'
@@ -103,6 +108,7 @@ u_box_bottom_left_arc = u'\u2570'
 u_box_horiz_light_heavy = u'\u257c'
 u_box_horiz_heavy_light = u'\u257e'
 u_skull_and_crossbones = u'\u2620'
+u_caduceus = u'\u2624'
 u_frowning_face = u'\u2639'
 u_smiling_face = u'\u263a'
 u_black_heart = u'\u2665'
@@ -111,12 +117,15 @@ u_male = u'\u2642'
 u_male_female = u'\u26a5'
 u_checkmark_thin = u'\u2713'
 u_checkmark_thick = u'\u2714'
+u_heavy_greek_cross = u'\u271a'
+u_arrow2right_thick = u'\u2794'
 u_writing_hand = u'\u270d'
 u_pencil_1 = u'\u270e'
 u_pencil_2 = u'\u270f'
 u_pencil_3 = u'\u2710'
 u_latin_cross = u'\u271d'
-u_kanji_yen = u'\u5186'						# Yen kanji
+u_arrow2right_until_black_diamond = u'\u291e'	# ->*
+u_kanji_yen = u'\u5186'							# Yen kanji
 u_replacement_character = u'\ufffd'
 u_link_symbol = u'\u1f517'
 
@@ -168,6 +177,29 @@ def rmdir(directory):
 		error_count += 1
 	return error_count
 
+#---------------------------------------------------------------------------
+def rm_dir_content(directory):
+	_log.debug('cleaning out [%s]', directory)
+	try:
+		items = os.listdir(directory)
+	except OSError:
+		return False
+	for item in items:
+		# attempt file/link removal and ignore (but log) errors
+		full_item = os.path.join(directory, item)
+		try:
+			os.remove(full_item)
+		except OSError:		# as per the docs, this is a directory
+			_log.debug('[%s] seems to be a subdirectory', full_item)
+			errors = rmdir(full_item)
+			if errors > 0:
+				return False
+		except StandardError:
+			_log.exception('cannot os.remove(%s) [a file or a link]', full_item)
+			return False
+
+	return True
+
 #---------------------------------------------------------------------------
 def mk_sandbox_dir(prefix=None, base_dir=None):
 	if prefix is None:
@@ -193,7 +225,12 @@ def dirname_stem(directory):
 
 #---------------------------------------------------------------------------
 def dir_is_empty(directory=None):
-	return len(os.listdir(directory)) == 0
+	try:
+		return len(os.listdir(directory)) == 0
+	except OSError as exc:
+		if exc.errno == 2:
+			return None
+		raise
 
 #---------------------------------------------------------------------------
 class gmPaths(gmBorg.cBorg):
@@ -600,20 +637,47 @@ def unicode_csv_reader(unicode_csv_data, dialect=csv.excel, encoding='utf-8', **
 			yield [ unicode(cell, encoding) for cell in row ]
 			#yield [unicode(cell, 'utf-8') for cell in row]
 
+#---------------------------------------------------------------------------
+def fname_sanitize(filename):
+	"""Normalizes unicode, removes non-alpha characters, converts spaces to underscores."""
+
+	dir_part, name_part = os.path.split(filename)
+	if name_part == u'':
+		return filename
+
+	import unicodedata
+	name_part = unicodedata.normalize('NFKD', name_part)
+	# remove everything not in group []
+	name_part = regex.sub (
+		'[^.\w\s[\]()%§+-]',
+		u'',
+		name_part,
+		flags = regex.UNICODE
+	).strip()
+	# translate whitespace to underscore
+	name_part = regex.sub (
+		'\s+',
+		u'_',
+		name_part,
+		flags = regex.UNICODE
+	)
+	return os.path.join(dir_part, name_part)
+
 #---------------------------------------------------------------------------
 def fname_stem(filename):
-	# /home/user/dir/filename.ext -> filename
+	"""/home/user/dir/filename.ext -> filename"""
 	return os.path.splitext(os.path.basename(filename))[0]
 
 #---------------------------------------------------------------------------
 def fname_stem_with_path(filename):
-	# /home/user/dir/filename.ext -> /home/user/dir/filename
+	"""/home/user/dir/filename.ext -> /home/user/dir/filename"""
 	return os.path.splitext(filename)[0]
 
 #---------------------------------------------------------------------------
 def fname_extension(filename=None, fallback=None):
-	# /home/user/dir/filename.ext -> .ext
-	# '' or '.' -> fallback if any else ''
+	"""	/home/user/dir/filename.ext -> .ext
+		'' or '.' -> fallback if any else ''
+	"""
 	ext = os.path.splitext(filename)[1]
 	if ext.strip() not in [u'.', u'']:
 		return ext
@@ -677,6 +741,43 @@ def get_unique_filename(prefix=None, suffix=None, tmp_dir=None, include_timestam
 
 	return filename
 
+#---------------------------------------------------------------------------
+def __make_symlink_on_windows(physical_name, link_name):
+	import ctypes
+	csl = ctypes.windll.kernel32.CreateSymbolicLinkW
+	csl.argtypes = (ctypes.c_wchar_p, ctypes.c_wchar_p, ctypes.c_uint32)
+	csl.restype = ctypes.c_ubyte
+	if os.path.isdir(physical_name):
+		flags = 1
+	else:
+		flags = 0
+	ret_code = csl(link_name, physical_name.replace('/', '\\'), flags)
+	_log.debug('ctypes.windll.kernel32.CreateSymbolicLinkW() exit code: %s', ret_code)
+	if ret_code == 0:
+		raise ctypes.WinError()
+	return ret_code
+
+#---------------------------------------------------------------------------
+def mklink(physical_name, link_name, overwrite=False):
+
+	_log.debug('creating symlink (overwrite = %s):', overwrite)
+	_log.debug('link [%s] =>', link_name)
+	_log.debug('=> physical [%s]', physical_name)
+
+	if os.path.exists(link_name):
+		_log.debug('link exists')
+		if overwrite:
+			return True
+		return False
+
+	try:
+		os.symlink(physical_name, link_name)
+	except AttributeError:
+		_log.debug('Windows does not have os.symlink(), resorting to ctypes')
+		__make_symlink_on_windows(physical_name, link_name)
+
+	return True
+
 #===========================================================================
 def import_module_from_directory(module_path=None, module_name=None, always_remove_path=False):
 	"""Import a module from any location."""
@@ -1054,6 +1155,31 @@ def shorten_text(text=None, max_length=None):
 
 	return text[:max_length-1] + u_ellipsis
 
+#---------------------------------------------------------------------------
+def shorten_words_in_line(text=None, max_length=None, min_word_length=None, ignore_numbers=True, ellipsis=u_ellipsis):
+	if text is None:
+		return None
+	if max_length is None:
+		max_length = len(text)
+	else:
+		if len(text) <= max_length:
+			return text
+	old_words = regex.split('\s+', text, flags = regex.UNICODE)
+	no_old_words = len(old_words)
+	max_word_length = max(min_word_length, (max_length // no_old_words))
+	words = []
+	for word in old_words:
+		if len(word) <= max_word_length:
+			words.append(word)
+			continue
+		if ignore_numbers:
+			tmp = word.replace(u'-', u'').replace(u'+', u'').replace(u'.', u'').replace(u',', u'').replace(u'/', u'').replace(u'&', u'').replace(u'*', u'')
+			if tmp.isdigit():
+				words.append(word)
+				continue
+		words.append(word[:max_word_length] + ellipsis)
+	return u' '.join(words)
+
 #---------------------------------------------------------------------------
 def xml_escape_string(text=None):
 	"""check for special XML characters and transform them"""
@@ -1297,7 +1423,7 @@ def format_dict_like(d, relevant_keys=None, template=None, missing_key_template=
 	else:
 		delim_left, delim_right = value_delimiters
 	if tabular:
-		max_len = max([ len(i) for i in relevant_keys ])
+		max_len = max([ len(u'%s' % k) for k in relevant_keys ])
 		max_len_str = u'%s.%s' % (max_len, max_len)
 		line_template = (u' ' * left_margin) + u'%' + max_len_str + ('s: %s%%s%s' % (delim_left, delim_right))
 	else:
@@ -1398,6 +1524,7 @@ def get_icon(wx=None):
 			_log.exception(u"this platform doesn't support wx.Icon().LoadFile()")
 
 	return icon
+
 #===========================================================================
 # main
 #---------------------------------------------------------------------------
@@ -1481,6 +1608,12 @@ if __name__ == '__main__':
 	#-----------------------------------------------------------------------
 	def test_coalesce():
 
+		val = None
+		print(val, coalesce(val, u'is None', u'is not None'))
+		val = 1
+		print(val, coalesce(val, u'is None', u'is not None'))
+		return
+
 		import datetime as dt
 		print(coalesce(initial = dt.datetime.now(), template_initial = u'-- %s --', function_initial = ('strftime', u'%Y-%m-%d')))
 
@@ -1734,6 +1867,7 @@ second line\n
 	#-----------------------------------------------------------------------
 	def test_dir_is_empty():
 		print(sys.argv[2], 'empty:', dir_is_empty(sys.argv[2]))
+
 	#-----------------------------------------------------------------------
 	def test_compare_dicts():
 		d1 = {}
@@ -1749,14 +1883,15 @@ second line\n
 		# 3
 		d2[4] = 4
 
-		compare_dict_likes(d1, d2)
+		#compare_dict_likes(d1, d2)
 
 		d1 = {1: 1, 2: 2}
 		d2 = {1: 1, 2: 2}
 
-		compare_dict_likes(d1, d2, 'same1', 'same2')
-		print(format_dict_like(d1))
-		print(format_dict_like(d2))
+		#compare_dict_likes(d1, d2, 'same1', 'same2')
+		print(format_dict_like(d1, tabular = False))
+		print(format_dict_like(d1, tabular = True))
+		#print(format_dict_like(d2))
 
 	#-----------------------------------------------------------------------
 	def test_format_compare_dicts():
@@ -1783,6 +1918,12 @@ second line\n
 	#-----------------------------------------------------------------------
 	def test_rm_dir():
 		rmdir('cx:\windows\system3__2xxxxxxxxxxxxx')
+
+	#-----------------------------------------------------------------------
+	def test_rm_dir_content():
+		#print(rm_dir_content('cx:\windows\system3__2xxxxxxxxxxxxx'))
+		print(rm_dir_content('/tmp/user/1000/tmp'))
+
 	#-----------------------------------------------------------------------
 	def test_strip_prefix():
 		tests = [
@@ -1814,9 +1955,19 @@ second line\n
 		]
 		for txt, lng in tst:
 			print(u'max', lng, 'of', txt, '=', shorten_text(txt, lng))
+	#-----------------------------------------------------------------------
+	def test_fname_sanitize():
+		tests = [
+			u'/tmp/test.txt',
+			u'/tmp/ test.txt',
+			u'/tmp/ tes\\t.txt',
+			u'test'
+		]
+		for test in tests:
+			print (test, fname_sanitize(test))
 
 	#-----------------------------------------------------------------------
-	#test_coalesce()
+	test_coalesce()
 	#test_capitalize()
 	#test_import_module()
 	#test_mkdir()
@@ -1840,8 +1991,10 @@ second line\n
 	#test_dir_is_empty()
 	#test_compare_dicts()
 	#test_rm_dir()
+	#test_rm_dir_content()
 	#test_strip_prefix()
 	#test_shorten_text()
-	test_format_compare_dicts()
+	#test_format_compare_dicts()
+	#test_fname_sanitize()
 
 #===========================================================================
