@@ -53,9 +53,11 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 			self.mode = 'edit'
 
 		#self.__init_ui()
+
 	#----------------------------------------------------------------
 #	def __init_ui(self):
 #		# adjust phrasewheels etc
+
 	#----------------------------------------------------------------
 	# generic Edit Area mixin API
 	#----------------------------------------------------------------
@@ -86,6 +88,7 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 			self._PRW_xxx.display_as_valid(True)
 
 		return validity
+
 	#----------------------------------------------------------------
 	def _save_as_new(self):
 
@@ -106,6 +109,7 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		self.data = data
 		return False
 		return True
+
 	#----------------------------------------------------------------
 	def _save_as_update(self):
 
@@ -118,15 +122,37 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		self.data[''] = self._CHBOX_xxx.GetValue()
 		self.data.save()
 		return True
+
 	#----------------------------------------------------------------
 	def _refresh_as_new(self):
 		pass
+
 	#----------------------------------------------------------------
 	def _refresh_as_new_from_existing(self):
 		self._refresh_as_new()
+
 	#----------------------------------------------------------------
 	def _refresh_from_existing(self):
 		pass
+
+	#----------------------------------------------------------------
+	def set_fields(self, fields):
+		# <fields> must be a dict compatible with the
+		# structure of the business object this edit
+		# area is for,
+		# thusly, the edit area knows how to set its
+		# controls from it,
+		# <fields> doesn't have to contain all keys, rather:
+		# - missing ones are skipped
+		# - unknown ones are ignored
+		# each key must hold a dict with at least a key 'value'
+		# and _can_ contain another key 'data',
+		# 'value' and 'data' must be compatible with the
+		# control they go into,
+		# controls which don't require 'data' (say, RadioButton)
+		# will ignore an existing 'data' key
+		pass
+
 	#----------------------------------------------------------------
 
 	**************** end of template ****************
@@ -140,6 +166,7 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 			False: 'pink'
 		}
 		self._refresh_as_new()
+
 	#----------------------------------------------------------------
 	# properties
 	#----------------------------------------------------------------
@@ -210,6 +237,7 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 
 		else:
 			raise ValueError('[%s] <mode> must be in %s' % (self.__class__.__name__, edit_area_modes))
+
 	#----------------------------------------------------------------
 	def refresh(self):
 		"""Invoked from the generic edit area dialog.
@@ -236,9 +264,11 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 			return result
 		else:
 			raise ValueError('[%s] <mode> must be in %s' % (self.__class__.__name__, edit_area_modes))
+
 	#----------------------------------------------------------------
 	def display_tctrl_as_valid(self, tctrl=None, valid=None):
 		self.display_ctrl_as_valid(ctrl = tctrl, valid = valid)
+
 	#----------------------------------------------------------------
 	def display_ctrl_as_valid(self, ctrl=None, valid=None):
 		ctrl.SetBackgroundColour(self.__tctrl_validity_colors[valid])
@@ -1511,7 +1541,7 @@ class gmPastHistoryEditArea(gmEditArea):
 		return values 
 
 	def _save_data(self):
-		clinical = self._patient.get_emr().get_past_history()
+		clinical = self._patient.emr.get_past_history()
 		if self.getDataId() is None:
 			id = clinical.create_history( self.get_fields_formatting_values() )
 			self.setDataId(id)
@@ -1674,7 +1704,7 @@ class gmPrescriptionEditArea(gmEditArea):
 		self.txt_problem = cEditAreaField(parent)
 		self.txt_class = cEditAreaField(parent)
 		self.txt_generic = cEditAreaField(parent)
-		self.txt_brand = cEditAreaField(parent)
+		self.txt_drug_product = cEditAreaField(parent)
 		self.txt_strength= cEditAreaField(parent)
 		self.txt_directions= cEditAreaField(parent)
 		self.txt_for = cEditAreaField(parent)
@@ -1683,7 +1713,7 @@ class gmPrescriptionEditArea(gmEditArea):
 		lines.append(self.txt_problem)
 		lines.append(self.txt_class)
 		lines.append(self.txt_generic)
-		lines.append(self.txt_brand)
+		lines.append(self.txt_drug_product)
 		lines.append(self.txt_strength)
 		lines.append(self.txt_directions)
 		lines.append(self.txt_for)
@@ -1693,7 +1723,7 @@ class gmPrescriptionEditArea(gmEditArea):
 			"problem": self.txt_problem,
 			"class" : self.txt_class,
 			"generic" : self.txt_generic,
-			"brand" : self.txt_brand,
+			"prod" : self.txt_drug_product,
 			"strength": self.txt_strength,
 			"directions": self.txt_directions,
 			"for" : self.txt_for,
@@ -1934,7 +1964,7 @@ class EditArea(wx.Panel):
 #		      self.text1_prescription_reason = cEditAreaField(self,-1,wx.DefaultPosition,wx.DefaultSize)
 #		      self.text2_drug_class = cEditAreaField(self,-1,wx.DefaultPosition,wx.DefaultSize)
 #		      self.text3_generic_drug = cEditAreaField(self,-1,wx.DefaultPosition,wx.DefaultSize)
-#		      self.text4_brand_drug = cEditAreaField(self,-1,wx.DefaultPosition,wx.DefaultSize)
+#		      self.text4_product_drug = cEditAreaField(self,-1,wx.DefaultPosition,wx.DefaultSize)
 #		      self.text5_strength = cEditAreaField(self,-1,wx.DefaultPosition,wx.DefaultSize)
 #		      self.text6_directions = cEditAreaField(self,-1,wx.DefaultPosition,wx.DefaultSize)
 #		      self.text7_for_duration = cEditAreaField(self,-1,wx.DefaultPosition,wx.DefaultSize)
@@ -1957,7 +1987,7 @@ class EditArea(wx.Panel):
 #		      self.sizer_line3.Add(self.text3_generic_drug,5,wx.EXPAND)
 #		      self.sizer_line3.Add(lbl_veterans,1,wx.EXPAND)
  #       	      self.sizer_line3.Add(self.cb_veteran,1,wx.EXPAND)
-#		      self.sizer_line4.Add(self.text4_brand_drug,5,wx.EXPAND)
+#		      self.sizer_line4.Add(self.text4_product_drug,5,wx.EXPAND)
 #		      self.sizer_line4.Add(lbl_reg24,1,wx.EXPAND)
  #       	      self.sizer_line4.Add(self.cb_reg24,1,wx.EXPAND)
 #		      self.sizer_line5.Add(self.text5_strength,5,wx.EXPAND)
@@ -1977,7 +2007,7 @@ class EditArea(wx.Panel):
 #		      self.gszr.Add(self.text1_prescription_reason,1,wx.EXPAND) #prescribe for
 #		      self.gszr.Add(self.text2_drug_class,1,wx.EXPAND) #prescribe by class
 #		      self.gszr.Add(self.sizer_line3,1,wx.EXPAND) #prescribe by generic, lbl_veterans, cb_veteran
-#		      self.gszr.Add(self.sizer_line4,1,wx.EXPAND) #prescribe by brand, lbl_reg24, cb_reg24
+#		      self.gszr.Add(self.sizer_line4,1,wx.EXPAND) #prescribe by product, lbl_reg24, cb_reg24
 #		      self.gszr.Add(self.sizer_line5,1,wx.EXPAND) #drug strength, lbl_quantity, text_quantity 
 #		      self.gszr.Add(self.sizer_line6,1,wx.EXPAND) #txt_directions, lbl_repeats, text_repeats 
 #		      self.gszr.Add(self.sizer_line7,1,wx.EXPAND) #text_for,lbl_usual,chk_usual
