@@ -216,7 +216,7 @@ class cNewPatientEAPnl(wxgNewPatientEAPnl.wxgNewPatientEAPnl, gmEditArea.cGeneri
 		self._PRW_urb.SetText(value = adr['urb'], data = adr['urb'])
 		self._PRW_urb.set_context(context = u'zip', val = adr['postcode'])
 
-		self._PRW_region.SetText(value = adr['l10n_state'], data = adr['code_state'])
+		self._PRW_region.SetText(value = adr['l10n_region'], data = adr['code_region'])
 		self._PRW_region.set_context(context = u'zip', val = adr['postcode'])
 
 		self._PRW_country.SetText(value = adr['l10n_country'], data = adr['code_country'])
@@ -259,6 +259,16 @@ class cNewPatientEAPnl(wxgNewPatientEAPnl.wxgNewPatientEAPnl, gmEditArea.cGeneri
 			error = True
 			self.display_ctrl_as_valid(ctrl = self._TCTRL_tob, valid = False)
 
+		# uniqueness
+		if gmPerson.this_person_exists (
+			self._PRW_lastname.GetValue().strip(),
+			self._PRW_firstnames.GetValue().strip(),
+			self._PRW_dob.GetData(),
+			gmTools.none_if(self._TCTRL_comment.GetValue().strip(), u'')
+		):
+			error = True
+			self.status_message = _('Duplicate person. Modify name and/or DOB or use comment to make unique.')
+
 		return (not error)
 	#----------------------------------------------------------------
 	def __address_valid_for_save(self, empty_address_is_valid=False):
@@ -384,6 +394,7 @@ class cNewPatientEAPnl(wxgNewPatientEAPnl.wxgNewPatientEAPnl, gmEditArea.cGeneri
 		return True
 	#----------------------------------------------------------------
 	def _on_leaving_dob(self):
+		_validate_dob_field(self._PRW_dob)
 		wx.CallAfter(self._refresh_dupe_warning)
 	#----------------------------------------------------------------
 	def _on_leaving_zip(self):
@@ -431,7 +442,7 @@ class cNewPatientEAPnl(wxgNewPatientEAPnl.wxgNewPatientEAPnl, gmEditArea.cGeneri
 		mapping = [
 			(self._PRW_street, 'street'),
 			(self._PRW_urb, 'urb'),
-			(self._PRW_region, 'l10n_state')
+			(self._PRW_region, 'l10n_region')
 		]
 		# loop through fields and invalidate address searcher if different
 		for ctrl, field in mapping:
@@ -515,7 +526,7 @@ class cNewPatientEAPnl(wxgNewPatientEAPnl.wxgNewPatientEAPnl, gmEditArea.cGeneri
 				_log.debug('street: >>%s<<', self._PRW_street.GetValue().strip())
 				_log.debug('postcode: >>%s<<', self._PRW_zip.GetValue().strip())
 				_log.debug('urb: >>%s<<', self._PRW_urb.GetValue().strip())
-				_log.debug('state: >>%s<<', self._PRW_region.GetData().strip())
+				_log.debug('region: >>%s<<', self._PRW_region.GetData().strip())
 				_log.debug('country: >>%s<<', self._PRW_country.GetData().strip())
 				_log.exception('cannot link address')
 				gmGuiHelpers.gm_show_error (
